## TODO: Implement wrapper to get grid and time info?

source("modules/Saving/paths2save.R")

get_times <- function(store.freq, fcst.horizon, leadtimes, sdate) {
  # Generates time dimensions and the corresponding metadata.
  ## TODO: Get correct indices according to the start date for monthly case.
  ## TODO: Format units according to standards.

  if (store.freq == "monthly_mean") {
    switch(fcst.horizon,
           "seasonal" = {time <- leadtimes; ref <- 'months since '; 
                         stdname <- paste(strtoi(leadtimes), collapse=", ")},
           "subseasonal" = {len <- 4; ref <- 'weeks since ';
                            stdname <- ''})
    # time <- 1:len ## Is this correct? I think it needs to be changed

  } else if (store.freq == "daily_mean") {
    switch(fcst.horizon,
	   "seasonal" = {time <- leadtimes; ref <- 'days since ';
	                 stdname <- paste(strtoi(leadtimes), collapse=", ")})
  }

  dim(time) <- length(time)
  # metadata <- list(time = list(standard_name = stdname,
  sdate <- as.Date(sdate, format = '%Y%m%d') # reformatting
  metadata <- list(time = list(units = paste0(ref, sdate, 'T00:00:00')))
  attr(time, 'variables') <- metadata
  names(dim(time)) <- 'time'
  
  time_step <- 1
  dim(time_step) <- length(time_step)
  metadata <- list(time_step = list(units = paste0(ref, sdate, 'T00:00:00')))
  attr(time_step, 'variables') <- metadata
  names(dim(time_step)) <- 'time_step'

  sdate <- 1:length(sdate)
  dim(sdate) <- length(sdate)
  metadata <- list(sdate = list(standard_name = paste(strtoi(sdate),
						      collapse=", "),
                                units = paste0('Init date')))
  attr(sdate, 'variables') <- metadata
  names(dim(sdate)) <- 'sdate'

  return(list(time_step=time_step, time=time, sdate=sdate))

}

get_latlon <- function(latitude, longitude) {
  # Adds dimensions and metadata to lat and lon
  # latitude: array containing the latitude values
  # longitude: array containing the longitude values
  
  dim(longitude) <- length(longitude)
  ## TODO: Extract metadata from s2dv_cube
  metadata <- list(longitude = list(units = 'degrees_east'))
  attr(longitude, 'variables') <- metadata
  names(dim(longitude)) <- 'longitude'
  
  dim(latitude) <- length(latitude)
  ## TODO: Extract metadata from s2dv_cube
  metadata <- list(latitude = list(units = 'degrees_north'))
  attr(latitude, 'variables') <- metadata
  names(dim(latitude)) <- 'latitude'

  return(list(lat=latitude, lon=longitude))

}

## TODO: Place inside a function somewhere
# if (tolower(agg) == "country"){
#   load(mask.path)
#   grid <- europe.countries.iso
# } else {
#   grid <- list(lon=attr(var.obs, 'Variables')$dat1$longitude,
#                lat=attr(var.obs, 'Variables')$dat1$latitude)
# }

save_metrics <- function(skill,
			 recipe,
                         data_cube,
                         outdir,
                         agg="global") {
  # This function adds metadata to the skill metrics in 'skill'
  # and exports them to a netCDF file inside 'outdir'.

  # Remove ensemble correlation from the list since it should be saved in
  # a separate file, as it has 'ensemble' dim.
  if ("enscorr" %in% names(skill)) {
    enscorr_metrics <- grep("enscorr", names(skill))
    skill <- skill[-enscorr_metrics]
  } 
  
  # Define grid dimensions and names
  lalo <- c('longitude', 'latitude')

  # Remove singleton dimensions and rearrange lon, lat and time dims 
  if (tolower(agg) == "global") {
    ## TODO: Implement metrics with additional non-singleton dimensions
    ## e.g. 'ensemble' in ensemble correlation.
    skill <- lapply(skill, function(x) {
                    Reorder(drop(x[[1]]), c(lalo, 'time'))})
  }

  for (i in 1:length(skill)) {
    ## TODO: create dictionary with proper metadata
    metric <- names(skill[i])
    if (tolower(agg) == "country") {
      sdname <- paste0(metric, " region-aggregated metric")
      dims <- c('Country', 'time')
    } else {
      sdname <- paste0(metric, " grid point metric") # formerly names(metric)
      dims <- c(lalo, 'time')
    }
    metadata <- list(metric = list(name = metric, standard_name = sdname))

    attr(skill[[i]], 'variables') <- metadata   
    names(dim(skill[[i]])) <- dims
  }

  # Time indices and metadata
  fcst.horizon <- tolower(recipe$Analysis$Horizon)
  store.freq <- recipe$Analysis$Variables$freq

  # Generate vector containing leadtimes
  if (store.freq == "monthly_mean") {
    leadtimes <- seq(from = recipe$Analysis$Time$leadtimemin,
		     to = recipe$Analysis$Time$leadtimemax)
  } else {
    dates <- data$hcst$Dates$start
    lubridate::year(dates) <- as.numeric(recipe$Analysis$Time$hcst_start)
    dates <- unique(as.Date(dates, format = '%Y-%m-%d'))
    init_date <- paste0(as.numeric(recipe$Analysis$Time$hcst_start),
			as.numeric(recipe$Analysis$Time$sdate$fcst_sday))
    init_date <- as.Date(init_date, format = '%Y%m%d')
    leadtimes <- as.numeric(dates - init_date)
  }

  # If a fcst is provided, use that as the ref. year. Otherwise use 1970.
  if (!is.null(recipe$Analysis$Time$sdate$fcst_syear)) {
    fcst.sdate <- paste0(recipe$Analysis$Time$sdate$fcst_syear,
			 recipe$Analysis$Time$sdate$fcst_sday)
  } else {
    fcst.sdate <- paste0("1970", recipe$Analysis$Time$sdate$fcst_sday)
  }

  times <- get_times(store.freq, fcst.horizon, leadtimes, fcst.sdate)
  time <- times$time
  time_step <- times$time_step

  ## TODO: Generate output filename
  outfile <- get_filename(outdir, data_cube$Variable$varName,
			  fcst.sdate, fcst.sdate,
			  agg, fcst.horizon, "skill")

  # Grid data and metadata
  if (tolower(agg) == "country") { 
    country <- get_countries(grid)
    ArrayToNc(append(country, time, skill, time_step), outfile)
  } else {
    latitude <- data_cube$lat[1:length(data_cube$lat)]
    longitude <- data_cube$lon[1:length(data_cube$lon)]
    latlon <- get_latlon(latitude, longitude)
    # Compile variables into a list and export to netCDF
    vars <- list(latlon$lat, latlon$lon, time)
    vars <- c(vars, skill, list(time_step))
    ArrayToNc(vars, outfile)
  }

  print("##### SKILL METRICS SAVED TO NETCDF FILE #####")

}

